CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
178 lines
6.0 KiB
C++
178 lines
6.0 KiB
C++
/*
|
|
* Copyright (c) Advanced Micro Devices, Inc., or its affiliates.
|
|
*
|
|
* SPDX-License-Identifier: MIT
|
|
*/
|
|
|
|
#include "memcpy_performance_common.hh"
|
|
|
|
/**
|
|
* @addtogroup memcpy memcpy
|
|
* @{
|
|
* @ingroup PerformanceTestMemory
|
|
*/
|
|
|
|
class Memcpy2DAsyncBenchmark : public Benchmark<Memcpy2DAsyncBenchmark> {
|
|
public:
|
|
void operator()(void* dst, size_t dst_pitch, const void* src, size_t src_pitch, size_t width,
|
|
size_t height, hipMemcpyKind kind, const hipStream_t& stream) {
|
|
TIMED_SECTION_STREAM(kTimerTypeEvent, stream) {
|
|
HIP_CHECK(hipMemcpy2DAsync(dst, dst_pitch, src, src_pitch, width, height, kind, stream));
|
|
}
|
|
HIP_CHECK(hipStreamSynchronize(stream));
|
|
}
|
|
};
|
|
|
|
static void RunBenchmark(size_t width, size_t height, hipMemcpyKind kind,
|
|
bool enable_peer_access = false) {
|
|
Memcpy2DAsyncBenchmark benchmark;
|
|
benchmark.AddSectionName("(" + std::to_string(width) + ", " + std::to_string(height) + ")");
|
|
|
|
const StreamGuard stream_guard(Streams::created);
|
|
const hipStream_t stream = stream_guard.stream();
|
|
|
|
if (kind == hipMemcpyDeviceToHost) {
|
|
LinearAllocGuard2D<int> device_allocation(width, height);
|
|
LinearAllocGuard<int> host_allocation(LinearAllocs::hipHostMalloc,
|
|
device_allocation.width() * height);
|
|
benchmark.Run(host_allocation.ptr(), device_allocation.width(), device_allocation.ptr(),
|
|
device_allocation.pitch(), device_allocation.width(), device_allocation.height(),
|
|
hipMemcpyDeviceToHost, stream);
|
|
} else if (kind == hipMemcpyHostToDevice) {
|
|
LinearAllocGuard2D<int> device_allocation(width, height);
|
|
LinearAllocGuard<int> host_allocation(LinearAllocs::hipHostMalloc,
|
|
device_allocation.width() * height);
|
|
benchmark.Run(device_allocation.ptr(), device_allocation.pitch(), host_allocation.ptr(),
|
|
device_allocation.width(), device_allocation.width(), device_allocation.height(),
|
|
hipMemcpyHostToDevice, stream);
|
|
} else if (kind == hipMemcpyHostToHost) {
|
|
LinearAllocGuard<int> src_allocation(LinearAllocs::hipHostMalloc, width * sizeof(int) * height);
|
|
LinearAllocGuard<int> dst_allocation(LinearAllocs::hipHostMalloc, width * sizeof(int) * height);
|
|
benchmark.Run(dst_allocation.ptr(), width * sizeof(int), src_allocation.ptr(),
|
|
width * sizeof(int), width * sizeof(int), height, hipMemcpyHostToHost, stream);
|
|
} else {
|
|
// hipMemcpyDeviceToDevice
|
|
int src_device = std::get<0>(GetDeviceIds(enable_peer_access));
|
|
int dst_device = std::get<1>(GetDeviceIds(enable_peer_access));
|
|
|
|
LinearAllocGuard2D<int> src_allocation(width, height);
|
|
HIP_CHECK(hipSetDevice(dst_device));
|
|
LinearAllocGuard2D<int> dst_allocation(width, height);
|
|
|
|
HIP_CHECK(hipSetDevice(src_device));
|
|
benchmark.Run(dst_allocation.ptr(), dst_allocation.pitch(), src_allocation.ptr(),
|
|
src_allocation.pitch(), dst_allocation.width(), dst_allocation.height(),
|
|
hipMemcpyDeviceToDevice, stream);
|
|
}
|
|
}
|
|
|
|
/**
|
|
* Test Description
|
|
* ------------------------
|
|
* - Executes `hipMemcpy2DAsync` from Device to Host:
|
|
* -# Allocation size
|
|
* - Small: 4 KB x 32 B
|
|
* - Medium: 4 MB x 32 B
|
|
* - Large: 16 MB x 32 B
|
|
* Test source
|
|
* ------------------------
|
|
* - performance/api/memcpy/hipMemcpy2DAsync.cc
|
|
* Test requirements
|
|
* ------------------------
|
|
* - HIP_VERSION >= 5.2
|
|
*/
|
|
HIP_TEST_CASE(Performance_hipMemcpy2DAsync_DeviceToHost) {
|
|
const auto width = GENERATE(4_KB, 4_MB, 16_MB);
|
|
RunBenchmark(width, 32, hipMemcpyDeviceToHost);
|
|
}
|
|
|
|
/**
|
|
* Test Description
|
|
* ------------------------
|
|
* - Executes `hipMemcpy2DAsync` from Host to Device:
|
|
* -# Allocation size
|
|
* - Small: 4 KB x 32 B
|
|
* - Medium: 4 MB x 32 B
|
|
* - Large: 16 MB x 32 B
|
|
* Test source
|
|
* ------------------------
|
|
* - performance/api/memcpy/hipMemcpy2DAsync.cc
|
|
* Test requirements
|
|
* ------------------------
|
|
* - HIP_VERSION >= 5.2
|
|
*/
|
|
HIP_TEST_CASE(Performance_hipMemcpy2DAsync_HostToDevice) {
|
|
const auto width = GENERATE(4_KB, 4_MB, 16_MB);
|
|
RunBenchmark(width, 32, hipMemcpyHostToDevice);
|
|
}
|
|
|
|
/**
|
|
* Test Description
|
|
* ------------------------
|
|
* - Executes `hipMemcpy2DAsync` from Host to Host:
|
|
* -# Allocation size
|
|
* - Small: 4 KB x 32 B
|
|
* - Medium: 4 MB x 32 B
|
|
* - Large: 16 MB x 32 B
|
|
* Test source
|
|
* ------------------------
|
|
* - performance/api/memcpy/hipMemcpy2DAsync.cc
|
|
* Test requirements
|
|
* ------------------------
|
|
* - HIP_VERSION >= 5.2
|
|
*/
|
|
HIP_TEST_CASE(Performance_hipMemcpy2DAsync_HostToHost) {
|
|
const auto width = GENERATE(4_KB, 4_MB, 16_MB);
|
|
RunBenchmark(width, 32, hipMemcpyHostToHost);
|
|
}
|
|
|
|
/**
|
|
* Test Description
|
|
* ------------------------
|
|
* - Executes `hipMemcpy2DAsync` from Device to Device with peer access disabled:
|
|
* -# Allocation size
|
|
* - Small: 4 KB x 32 B
|
|
* - Medium: 4 MB x 32 B
|
|
* - Large: 16 MB x 32 B
|
|
* Test source
|
|
* ------------------------
|
|
* - performance/api/memcpy/hipMemcpy2DAsync.cc
|
|
* Test requirements
|
|
* ------------------------
|
|
* - HIP_VERSION >= 5.2
|
|
*/
|
|
HIP_TEST_CASE(Performance_hipMemcpy2DAsync_DeviceToDevice_DisablePeerAccess) {
|
|
const auto width = GENERATE(4_KB, 4_MB, 16_MB);
|
|
RunBenchmark(width, 32, hipMemcpyDeviceToDevice);
|
|
}
|
|
|
|
/**
|
|
* Test Description
|
|
* ------------------------
|
|
* - Executes `hipMemcpy2D` from Device to Device with peer access enabled:
|
|
* -# Allocation size
|
|
* - Small: 4 KB x 32 B
|
|
* - Medium: 4 MB x 32 B
|
|
* - Large: 16 MB x 32 B
|
|
* Test source
|
|
* ------------------------
|
|
* - performance/api/memcpy/hipMemcpy2DAsync.cc
|
|
* Test requirements
|
|
* ------------------------
|
|
* - Multi-device
|
|
* - Device supports Peer-to-Peer access
|
|
* - HIP_VERSION >= 5.2
|
|
*/
|
|
HIP_TEST_CASE(Performance_hipMemcpy2DAsync_DeviceToDevice_EnablePeerAccess) {
|
|
if (HipTest::getDeviceCount() < 2) {
|
|
HIP_SKIP_TEST(HipTest::SkipReason::kFewerThanTwoGpus);
|
|
}
|
|
const auto width = GENERATE(4_KB, 4_MB, 16_MB);
|
|
RunBenchmark(width, 32, hipMemcpyDeviceToDevice, true);
|
|
}
|
|
|
|
/**
|
|
* End doxygen group memcpy.
|
|
* @}
|
|
*/
|